/*
 * direct.h - CELLFS direct block pointer.
 *
 * Copyright (C) 2006-2008 Nippon Telegraph and Telephone Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Written by Koji Sato <koji@osrg.net>.
 */

#ifndef _CELLFS_DIRECT_H
#define _CELLFS_DIRECT_H

#include <linux/types.h>
#include <linux/buffer_head.h>
#include "bmap.h"


/**
 * struct cellfs_direct_node - direct node
 * @dn_flags: flags
 * @dn_pad: padding
 */
struct cellfs_direct_node {
	__u8 dn_flags;
	__u8 pad[7];
};

#define CELLFS_DIRECT_NBLOCKS	(CELLFS_BMAP_SIZE / sizeof(__le64) - 1)
#define CELLFS_DIRECT_KEY_MIN	0
#define CELLFS_DIRECT_KEY_MAX	(CELLFS_DIRECT_NBLOCKS - 1)


int cellfs_direct_init(struct cellfs_bmap *);
int cellfs_direct_delete_and_convert(struct cellfs_bmap *, __u64, __u64 *,
				    __u64 *, int);


#endif	/* _CELLFS_DIRECT_H */
