/*
 * page.h - buffer/page management specific to CELLFS
 *
 * Copyright (C) 2005-2008 Nippon Telegraph and Telephone Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Written by Ryusuke Konishi <ryusuke@osrg.net>,
 *            Seiji Kihara <kihara@osrg.net>.
 */

#ifndef _CELLFS_PAGE_H
#define _CELLFS_PAGE_H

#include <linux/buffer_head.h>
#include "cellfs.h"

/*
 * Extended buffer state bits
 */
enum {
	BH_CELLFS_Allocated = BH_PrivateStart,
	BH_CELLFS_Node,
	BH_CELLFS_Volatile,
	BH_CELLFS_Checked,
	BH_CELLFS_Redirected,
};

BUFFER_FNS(CELLFS_Node, cellfs_node)		/* cellfs node buffers */
BUFFER_FNS(CELLFS_Volatile, cellfs_volatile)
BUFFER_FNS(CELLFS_Checked, cellfs_checked)	/* buffer is verified */
BUFFER_FNS(CELLFS_Redirected, cellfs_redirected)	/* redirected to a copy */


int __cellfs_clear_page_dirty(struct page *);

struct buffer_head *cellfs_grab_buffer(struct inode *, struct address_space *,
				      unsigned long, unsigned long);
void cellfs_forget_buffer(struct buffer_head *);
void cellfs_copy_buffer(struct buffer_head *, struct buffer_head *);
int cellfs_page_buffers_clean(struct page *);
void cellfs_page_bug(struct page *);

int cellfs_copy_dirty_pages(struct address_space *, struct address_space *);
void cellfs_copy_back_pages(struct address_space *, struct address_space *);
void cellfs_clear_dirty_page(struct page *, bool);
void cellfs_clear_dirty_pages(struct address_space *, bool);
void cellfs_mapping_init(struct address_space *mapping, struct inode *inode,
			struct backing_dev_info *bdi);
unsigned cellfs_page_count_clean_buffers(struct page *, unsigned, unsigned);
unsigned long cellfs_find_uncommitted_extent(struct inode *inode,
					    sector_t start_blk,
					    sector_t *blkoff);

#define CELLFS_PAGE_BUG(page, m, a...) \
	do { cellfs_page_bug(page); BUG(); } while (0)

static inline struct buffer_head *
cellfs_page_get_nth_block(struct page *page, unsigned int count)
{
	struct buffer_head *bh = page_buffers(page);

	while (count-- > 0)
		bh = bh->b_this_page;
	get_bh(bh);
	return bh;
}

#endif /* _CELLFS_PAGE_H */
