/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * CELLFS B-tree node cache
 *
 * Copyright (C) 2005-2008 Nippon Telegraph and Telephone Corporation.
 *
 * Written by Seiji Kihara.
 * Revised by Ryusuke Konishi.
 */

#ifndef _CELLFS_BTNODE_H
#define _CELLFS_BTNODE_H

#include <linux/types.h>
#include <linux/buffer_head.h>
#include <linux/fs.h>
#include <linux/backing-dev.h>

/**
 * struct cellfs_btnode_chkey_ctxt - change key context
 * @oldkey: old key of block's moving content
 * @newkey: new key for block's content
 * @bh: buffer head of old buffer
 * @newbh: buffer head of new buffer
 */
struct cellfs_btnode_chkey_ctxt {
	__u64 oldkey;
	__u64 newkey;
	struct buffer_head *bh;
	struct buffer_head *newbh;
};

void cellfs_init_btnc_inode(struct inode *btnc_inode);
void cellfs_btnode_cache_clear(struct address_space *);
struct buffer_head *cellfs_btnode_create_block(struct address_space *btnc,
					      __u64 blocknr);
int cellfs_btnode_submit_block(struct address_space *, __u64, sector_t, int,
			      int, struct buffer_head **, sector_t *);
void cellfs_btnode_delete(struct buffer_head *);
int cellfs_btnode_prepare_change_key(struct address_space *,
				    struct cellfs_btnode_chkey_ctxt *);
void cellfs_btnode_commit_change_key(struct address_space *,
				    struct cellfs_btnode_chkey_ctxt *);
void cellfs_btnode_abort_change_key(struct address_space *,
				   struct cellfs_btnode_chkey_ctxt *);

#endif	/* _CELLFS_BTNODE_H */
