/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * cpfile.h - CELLFS checkpoint file.
 *
 * Copyright (C) 2006-2008 Nippon Telegraph and Telephone Corporation.
 *
 * Written by Koji Sato.
 */

#ifndef _CELLFS_CPFILE_H
#define _CELLFS_CPFILE_H

#include <linux/fs.h>
#include <linux/buffer_head.h>
#include <linux/cellfs_api.h>		/* cellfs_cpstat */
#include <linux/cellfs_ondisk.h>	/* cellfs_inode, cellfs_checkpoint */


int cellfs_cpfile_get_checkpoint(struct inode *, __u64, int,
				struct cellfs_checkpoint **,
				struct buffer_head **);
void cellfs_cpfile_put_checkpoint(struct inode *, __u64, struct buffer_head *);
int cellfs_cpfile_delete_checkpoints(struct inode *, __u64, __u64);
int cellfs_cpfile_delete_checkpoint(struct inode *, __u64);
int cellfs_cpfile_change_cpmode(struct inode *, __u64, int);
int cellfs_cpfile_is_snapshot(struct inode *, __u64);
int cellfs_cpfile_get_stat(struct inode *, struct cellfs_cpstat *);
ssize_t cellfs_cpfile_get_cpinfo(struct inode *, __u64 *, int, void *,
				unsigned int, size_t);

int cellfs_cpfile_read(struct super_block *sb, size_t cpsize,
		      struct cellfs_inode *raw_inode, struct inode **inodep);

#endif	/* _CELLFS_CPFILE_H */
