/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * CELLFS disk address translation.
 *
 * Copyright (C) 2006-2008 Nippon Telegraph and Telephone Corporation.
 *
 * Written by Koji Sato.
 */

#ifndef _CELLFS_DAT_H
#define _CELLFS_DAT_H

#include <linux/types.h>
#include <linux/buffer_head.h>
#include <linux/fs.h>
#include <linux/cellfs_ondisk.h>	/* cellfs_inode, cellfs_checkpoint */


struct cellfs_palloc_req;

int cellfs_dat_translate(struct inode *, __u64, sector_t *);

int cellfs_dat_prepare_alloc(struct inode *, struct cellfs_palloc_req *);
void cellfs_dat_commit_alloc(struct inode *, struct cellfs_palloc_req *);
void cellfs_dat_abort_alloc(struct inode *, struct cellfs_palloc_req *);
int cellfs_dat_prepare_start(struct inode *, struct cellfs_palloc_req *);
void cellfs_dat_commit_start(struct inode *, struct cellfs_palloc_req *,
			    sector_t);
int cellfs_dat_prepare_end(struct inode *, struct cellfs_palloc_req *);
void cellfs_dat_commit_end(struct inode *, struct cellfs_palloc_req *, int);
void cellfs_dat_abort_end(struct inode *, struct cellfs_palloc_req *);
int cellfs_dat_prepare_update(struct inode *, struct cellfs_palloc_req *,
			     struct cellfs_palloc_req *);
void cellfs_dat_commit_update(struct inode *, struct cellfs_palloc_req *,
			     struct cellfs_palloc_req *, int);
void cellfs_dat_abort_update(struct inode *, struct cellfs_palloc_req *,
			    struct cellfs_palloc_req *);

int cellfs_dat_mark_dirty(struct inode *, __u64);
int cellfs_dat_freev(struct inode *, __u64 *, size_t);
int cellfs_dat_move(struct inode *, __u64, sector_t);
ssize_t cellfs_dat_get_vinfo(struct inode *, void *, unsigned int, size_t);

int cellfs_dat_read(struct super_block *sb, size_t entry_size,
		   struct cellfs_inode *raw_inode, struct inode **inodep);

#endif	/* _CELLFS_DAT_H */
