/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * CELLFS inode file
 *
 * Copyright (C) 2006-2008 Nippon Telegraph and Telephone Corporation.
 *
 * Written by Amagai Yoshiji.
 * Revised by Ryusuke Konishi.
 *
 */

#ifndef _CELLFS_IFILE_H
#define _CELLFS_IFILE_H

#include <linux/fs.h>
#include <linux/buffer_head.h>
#include "mdt.h"
#include "alloc.h"


static inline struct cellfs_inode *
cellfs_ifile_map_inode(struct inode *ifile, ino_t ino, struct buffer_head *ibh)
{
	void *kaddr = kmap(ibh->b_page);

	return cellfs_palloc_block_get_entry(ifile, ino, ibh, kaddr);
}

static inline void cellfs_ifile_unmap_inode(struct inode *ifile, ino_t ino,
					   struct buffer_head *ibh)
{
	kunmap(ibh->b_page);
}

int cellfs_ifile_create_inode(struct inode *, ino_t *, struct buffer_head **);
int cellfs_ifile_delete_inode(struct inode *, ino_t);
int cellfs_ifile_get_inode_block(struct inode *, ino_t, struct buffer_head **);

int cellfs_ifile_count_free_inodes(struct inode *, u64 *, u64 *);

int cellfs_ifile_read(struct super_block *sb, struct cellfs_root *root,
		     size_t inode_size, struct cellfs_inode *raw_inode,
		     struct inode **inodep);

#endif	/* _CELLFS_IFILE_H */
