/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Buffer/page management specific to CELLFS
 *
 * Copyright (C) 2005-2008 Nippon Telegraph and Telephone Corporation.
 *
 * Written by Ryusuke Konishi and Seiji Kihara.
 */

#ifndef _CELLFS_PAGE_H
#define _CELLFS_PAGE_H

#include <linux/buffer_head.h>
#include "cellfs.h"

/*
 * Extended buffer state bits
 */
enum {
	BH_CELLFS_Allocated = BH_PrivateStart,
	BH_CELLFS_Node,
	BH_CELLFS_Volatile,
	BH_CELLFS_Checked,
	BH_CELLFS_Redirected,
};

BUFFER_FNS(CELLFS_Node, cellfs_node)		/* cellfs node buffers */
BUFFER_FNS(CELLFS_Volatile, cellfs_volatile)
BUFFER_FNS(CELLFS_Checked, cellfs_checked)	/* buffer is verified */
BUFFER_FNS(CELLFS_Redirected, cellfs_redirected)	/* redirected to a copy */


void __cellfs_clear_folio_dirty(struct folio *);

struct buffer_head *cellfs_grab_buffer(struct inode *, struct address_space *,
				      unsigned long, unsigned long);
void cellfs_forget_buffer(struct buffer_head *);
void cellfs_copy_buffer(struct buffer_head *, struct buffer_head *);
bool cellfs_folio_buffers_clean(struct folio *);
void cellfs_folio_bug(struct folio *);

int cellfs_copy_dirty_pages(struct address_space *, struct address_space *);
void cellfs_copy_back_pages(struct address_space *, struct address_space *);
void cellfs_clear_folio_dirty(struct folio *, bool);
void cellfs_clear_dirty_pages(struct address_space *, bool);
unsigned int cellfs_page_count_clean_buffers(struct page *, unsigned int,
					    unsigned int);
unsigned long cellfs_find_uncommitted_extent(struct inode *inode,
					    sector_t start_blk,
					    sector_t *blkoff);

#define CELLFS_FOLIO_BUG(folio, m, a...) \
	do { cellfs_folio_bug(folio); BUG(); } while (0)

#endif /* _CELLFS_PAGE_H */
